Barrel = LibStub("AceAddon-3.0"):NewAddon("Barrel", "AceConsole-3.0", "AceEvent-3.0")
local ldb = LibStub("LibDataBroker-1.1")
local L = LibStub("AceLocale-3.0"):GetLocale("Barrel")

function Barrel:OnInitialize()

	-- Called when the addon is loaded
	self:Config()
	self:RegisterChatCommand("br", "ChatCommand")
	self:RegisterChatCommand("barrel", "ChatCommand")
	self:RegisterEvent("PLAYER_LOGIN")

end

function Barrel:PLAYER_LOGIN()
	for name, dataobj in ldb:DataObjectIterator() do
		local name = tostring(name).."Minimap"
		
		if not self.db.profile[name] then
			self.db.profile[name] = {}
			self.db.profile[name].radius = 80
			self.db.profile[name].rounding = 10
			self.db.profile[name].angle = 282
			self.db.profile[name].Hidden = false
			self.db.profile[name].dragable = true
		end
		
		-- MinimapFrame
		local f = CreateFrame("Frame", name, Minimap)
		f:SetFrameStrata("LOW")
		f:SetWidth(33)
		f:SetHeight(33)
		f:SetPoint("CENTER")
		f:EnableMouse(true)
		f:Show()
		--self.Minimap = f
		self[name] = f
		
		-- Minimap
		local b = CreateFrame("Button", nil, f)
		b:SetAllPoints(f)
		b:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
		
		-- MinimapIcon
		local t = b:CreateTexture(nil, "BACKGROUND")
		t:SetWidth(20)
		t:SetHeight(20)
		t:SetPoint("CENTER")
		MinimapIconTexture[name] = t
		MinimapIconTexture[name]:SetTexture(dataobj.icon)
		
		-- MinimapBorder
		t = b:CreateTexture(nil, "OVERLAY")
		t:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder")
		t:SetWidth(52)
		t:SetHeight(52)
		t:SetPoint("TOPLEFT")
		
		-- set some scripts
		b:SetScript("OnDragStart", function()
				if(self.db.profile[name].dragable) then
					this.dragme = true
					this:LockHighlight()
				end
			end)
		b:SetScript("OnDragStop", function()
				this.dragme = false
				this:UnlockHighlight()
			end)
		b:SetScript("OnUpdate", function()
				if(this.dragme == true) then
					self:MinimapBeingDragged(name)
				end
			end)
		b:SetScript("OnEnter", function()
			GameTooltip:SetOwner(this, "ANCHOR_NONE")
			GameTooltip:SetPoint("TOPLEFT", this, "BOTTOMLEFT")
			GameTooltip:ClearLines()
			if dataobj.OnEnter then
				dataobj.OnEnter(this)
			end
			if dataobj.OnTooltipShow then
				dataobj.OnTooltipShow(GameTooltip)
				GameTooltip:Show()
			elseif dataobj.tooltip then
				dataobj.tooltip:Show()
				if dataobj.tooltiptext then
					dataobj.tooltip:SetText(dataobj.tooltiptext)
				end
			elseif dataobj.tooltiptext then
				GameTooltip:SetText(dataobj.tooltiptext)
				GameTooltip:Show()
			end
			
		end)
		if dataobj.OnLeave then
			b:SetScript("OnLeave", dataobj.OnLeave
			)
		else
			b:SetScript("OnLeave", function() GameTooltip:Hide() end)
		end
		
		if dataobj.OnClick then
			b:SetScript("OnClick", dataobj.OnClick)
		end
		
		-- init dragging
		b:RegisterForDrag("LeftButton")
		b.dragme = false
		
		b:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		
		-- update the position of the button
		self:MinimapUpdatePosition(name)
		
		if ( self.db.profile[name].Hidden == true ) then
			self:HideMinimap()
		end
	end
	self:IterateBroker()
	ldb.RegisterCallback(self, "LibDataBroker_AttributeChanged__icon", "AttributeChanged_icon")
end

function Barrel:OnEnable()
end

function Barrel:ChatCommand(input)

	local input = input

	if not input or input:trim() == "" then
	  LibStub("AceConfigDialog-3.0"):Open("Barrel")
	else
	  LibStub("AceConfigCmd-3.0").HandleCommand(Barrel, "br", "Barrel", input)
	end

end

function Barrel:SetMinimapPosition(name, angle, radius, rounding)
	local name = name
	self.db.profile[name].angle = angle
	if(radius) then
		self.db.profile[name].radius = radius
	end
	if(rounding) then
		self.db.profile[name].rounding = rounding
	end
	self:MinimapUpdatePosition(name)
	self:OnMinimapMove(angle)
end

function Barrel:ShowMinimap(name)
	self[name]:Show()
end

function Barrel:HideMinimap(name)
	self[name]:Hide()
end

function Barrel:OnMinimapMove(angle)
end

function Barrel:MinimapBeingDragged(name)
	local name = name
	local mx, my = Minimap:GetCenter()
	local mz = MinimapCluster:GetScale()
	local cx, cy = GetCursorPosition(UIParent)
	local cz = UIParent:GetEffectiveScale()
	local v = math.deg(math.atan2(cy / cz - my * mz, cx / cz - mx * mz))
	if v < 0 then
		v = v + 360
	elseif v > 360 then
		v = v - 360
	end
	self:SetMinimapPosition(name, v)
end

local MinimapShapes = {
	-- quadrant booleans (same order as SetTexCoord)
	-- {upper-left, lower-left, upper-right, lower-right}
	-- true = rounded, false = squared
	["ROUND"] 					= {true, true, true, true},
	["SQUARE"] 					= {false, false, false, false},
	["CORNER-TOPLEFT"] 			= {true, false, false, false},
	["CORNER-TOPRIGHT"] 		= {false, false, true, false},
	["CORNER-BOTTOMLEFT"] 		= {false, true, false, false},
	["CORNER-BOTTOMRIGHT"]	 	= {false, false, false, true},
	["SIDE-LEFT"] 				= {true, true, false, false},
	["SIDE-RIGHT"] 				= {false, false, true, true},
	["SIDE-TOP"] 				= {true, false, true, false},
	["SIDE-BOTTOM"] 			= {false, true, false, true},
	["TRICORNER-TOPLEFT"] 		= {true, true, true, false},
	["TRICORNER-TOPRIGHT"] 		= {true, false, true, true},
	["TRICORNER-BOTTOMLEFT"] 	= {true, true, false, true},
	["TRICORNER-BOTTOMRIGHT"] 	= {false, true, true, true},
	}

function Barrel:MinimapUpdatePosition(name)
	local name = name
	
	local radius = self.db.profile[name].radius
	local rounding = self.db.profile[name].rounding
	local angle = math.rad(self.db.profile[name].angle)
	
	local x = math.cos(angle)
	local y = math.sin(angle)
	local q = 1;
	if x < 0 then
		q = q + 1;	-- lower
	end
	if y > 0 then
		q = q + 2;	-- right
	end
	local minimapShape = GetMinimapShape and GetMinimapShape() or "ROUND"
	local quadTable = MinimapShapes[minimapShape]
	if quadTable[q] then
		x = x*radius
		y = y*radius
	else
		local diagRadius = math.sqrt(2*(radius)^2)-rounding
		x = math.max(-radius, math.min(x*diagRadius, radius))
		y = math.max(-radius, math.min(y*diagRadius, radius))
	end
	self[name]:SetPoint("CENTER", Minimap, "CENTER", x, y-1)
end

function Barrel:AttributeChanged_icon(event, name, key, value, dataobj)
	local name = tostring(name).."Minimap"
	MinimapIconTexture[name]:SetTexture(dataobj.icon)
end



-- Copyright 2008 Copystring"